@@ -59,18 +59,15 @@ typedef struct {
 								   of numbers (enumerated type) */
 } CNUM;
 
-typedef struct {
-	CODE code;					/* CFG_XXX */
-	const char *extra_val;		/* if defined - name of special value
-								   (represented as "") */
-	char *initial;
-	char current[CFGVALUE_LEN + 1], new[CFGVALUE_LEN + 1];
-} CSTR;
-
 static CNUM table_numeric[] = {
 	/* enumerated */
 	{ CFG_CMD_LINES,	0, 2, 4, 2, 0, 0,
-		{	"2 screen lines", "3 screen lines", "4 screen lines" } },
+		{	"2 screen lines",
+			"3 screen lines",
+			"4 screen lines" } },
+	{ CFG_KILOBYTE,	0, 0, 1, 0, 0, 0,
+		{	"1 KiB is 1024 bytes (IEC standard)",
+			"1 KB is 1000 bytes (SI standard)" } },
 	{ CFG_FMT_NUMBER,	0, 0, 2, 0, 0, 0,
 		{	"AUTO (according to the current locale)",
 			"1.000.000 (dot separated)",
@@ -108,6 +105,14 @@ static CNUM table_numeric[] = {
 	{ CFG_H_SIZE,		0,		10, 100, 40, 0, 0, { 0 } }
 };
 
+typedef struct {
+	CODE code;					/* CFG_XXX */
+	const char *extra_val;		/* if defined - name of special value
+								   (represented as "") */
+	char *initial;
+	char current[CFGVALUE_LEN + 1], new[CFGVALUE_LEN + 1];
+} CSTR;
+
 static CSTR table_string[] = {
 	{ CFG_CMD_F3,		0,		"more $f",		"", "" },
 	{ CFG_CMD_F4,		0,		"vi $f",		"", "" },
@@ -134,13 +139,13 @@ static CSTR table_string[] = {
 };
 
 /*
- * help - everything in one place and sorted for easy editing,
+ * help - everything in one place and alphabetically sorted for easy editing,
  * text should fit on minimal width screen
  */
 static struct {
 	CODE code;
 	char *help;
-} table_help[] = {
+} table_help[CFG_VARIABLES] = {
 	{ CFG_C_SIZE,		"Completion panel size (AUTO = screen size)" },
 	{ CFG_CMD_F3,		"Command F3 = view file(s)" },
 	{ CFG_CMD_F4,		"Command F4 = edit file(s)" },
@@ -154,6 +159,7 @@ static struct {
 	{ CFG_CMD_F12,		"Command F12 = user defined" },
 	{ CFG_CMD_LINES,	"Appearance: "
 		"How many lines are occupied by the input line" },
+	{ CFG_COLLATION,	"Appearance: Filename collation sequence" },
 	{ CFG_D_SIZE,		"Directory panel size (AUTO = screen size)" },
 	{ CFG_DIR2,			"Secondary file panel's directory "
 						"(HOME = home directory)" },
@@ -163,12 +169,12 @@ static struct {
 		"Format: date (e.g. dMy, y/m/d), or AUTO" },
 	{ CFG_FMT_NUMBER,	"Appearance: Format: thousands separator" },
 	{ CFG_FMT_TIME,		"Appearance: Format: 12/24 hour clock" },
-	{ CFG_COLLATION,	"Appearance: Filename collation sequence" },
 	{ CFG_GROUP_FILES,	"Appearance: "
 		"Group files of the same type together" },
 	{ CFG_H_SIZE,		"History panel size" },
 	{ CFG_HELPFILE,		"External helpfile "
 						"(NONE = use built-in help instead)" },
+	{ CFG_KILOBYTE,		"Appearance: Filesize unit definition" },
 	{ CFG_LAYOUT,		"Appearance: "
 		"Which file panel layout is active" },
 	{ CFG_LAYOUT1,		"Appearance: File panel layout #1, see help" },
@@ -188,7 +194,8 @@ static struct {
 		"Change the X terminal window title" }
 };
 
-static CONFIG_ENTRY config[] = {
+static CONFIG_ENTRY config[CFG_VARIABLES] = {
+	/* do not exceed max length of CFGVAR_LEN */
 	{ "FRAME",			0,0,0,0,0 },
 	{ "CMD_LINES",		0,0,0,0,0 },
 	{ "XTERM_TITLE",	0,0,0,0,0 },
@@ -198,6 +205,7 @@ static CONFIG_ENTRY config[] = {
 	{ "LAYOUT2",		0,0,0,0,0 },
 	{ "LAYOUT3",		0,0,0,0,0 },
 	{ "ACTIVE_LAYOUT",	0,0,0,0,0 },
+	{ "KILOBYTE",		0,0,0,0,0 },
 	{ "FMT_NUMBER",		0,0,0,0,0 },
 	{ "FMT_TIME",		0,0,0,0,0 },
 	{ "FMT_DATE",		0,0,0,0,0 },
@@ -279,7 +287,7 @@ get_variable(const char *var)
 {
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(config); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		if (strcmp(config[i].var,var) == 0)
 			return config + i;
 	return 0;
@@ -313,7 +321,7 @@ get_help(int code)
 	int i;
 	const char *helpstr;
 
-	for (i = 0; i < ARRAY_SIZE(table_help); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		if (table_help[i].code == code) {
 			helpstr = table_help[i].help;
 			if (strlen(helpstr) > MIN_COLS - 4)
@@ -452,13 +460,20 @@ void
 config_initialize(void)
 {
 	int i, fd;
+	CNUM *pnum;
+	CSTR *pstr;
 
-	/* initialize 'config' table */
-	for (i = 0; i < ARRAY_SIZE(config); i++) {
-		if ( (config[i].table = get_numeric(i)) )
+	/* initialize 'config' & 'pcfg' tables */
+	for (i = 0; i < CFG_VARIABLES; i++) {
+		if ( (pnum = get_numeric(i)) ) {
 			config[i].isnum = 1;
-		else if ( (config[i].table = get_string(i)) )
-			config[i].isnum = 0;
+			config[i].table = pnum;
+			pcfg[i] = &pnum->current;
+		}
+		else if ( (pstr = get_string(i)) ) {
+			config[i].table = pstr;
+			pcfg[i] = &pstr->current;
+		}
 		else
 			err_exit("BUG: config variable not defined (code %d)",i);
 		if ( (config[i].help = get_help(i)) == 0)
@@ -466,7 +481,8 @@ config_initialize(void)
 	}
 
 	if (clex_data.admin) {
-		panel_cfg.extra_lines = 2;
+		panel_cfg.pd->min = -2;
+		panel_cfg.pd->type = PANEL_TYPE_CFG_ADMIN;
 		/*
 		 * check user's access rights
 		 * this may create an empty file, never mind
@@ -482,57 +498,27 @@ config_initialize(void)
 		close(fd);
 
 		/* initialize, read defaults */
-		for (i = 0; i < ARRAY_SIZE(config); i++)
+		for (i = 0; i < CFG_VARIABLES; i++)
 			set_value(i,MOV_I2C);
 		read_configuration(CONFIG_FILE,0);
 	}
 	else {
-		panel_cfg.extra_lines = 3;
+		panel_cfg.pd->min = -3;
+		panel_cfg.pd->type = PANEL_TYPE_CFG;
 		/* user's configuration file name */
 		pathname_set_directory(clex_data.homedir);
 		user_config_file = estrdup(pathname_join(".clexrc"));
 
 		/* read defaults, initialize, read config */
 		read_configuration(CONFIG_FILE,1);
-		for (i = 0; i < ARRAY_SIZE(config); i++)
+		for (i = 0; i < CFG_VARIABLES; i++)
 			set_value(i,MOV_I2C);
 		read_configuration(user_config_file,0);
 	}
 
-	panel_cfg.pd->cnt = panel_cfg.extra_lines + ARRAY_SIZE(config);
 	panel_cfg.config = config;
 }
 
-int
-config_num(int code)
-{
-	if (!config[code].isnum)
-		err_exit("BUG: config variable type mismatch (code %d)",code);
-	return ((CNUM *)config[code].table)->current;
-}
-
-/*
- * returned pointer can be cached for further reference
- * to the same variable
- *
- * CFG_LAYOUT is a special case, the active layout from
- * three available layouts is returned !
- */
-const char *
-config_str(int code)
-{
-	if (config[code].isnum) {
-		if (code == CFG_LAYOUT)
-			switch (config_num(CFG_LAYOUT)) {
-				case 0: return config_str(CFG_LAYOUT1);
-				case 1: return config_str(CFG_LAYOUT2);
-				case 2: return config_str(CFG_LAYOUT3);
-			}
-		err_exit("BUG: config variable type mismatch (code %d)",code);
-	}
-	return ((CSTR *)config[code].table)->current;
-}
-
 /* the 'new' value in readable text form */
 const char *
 config_print_value(int i)
@@ -585,7 +571,7 @@ config_save(void)
 	fprintf(fp,	"#\n"
 				"# CLEX configuration file\n"
 				"#\n");
-	for (i = 0; i < ARRAY_SIZE(config); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		if (config[i].saveit) {
 			if (config[i].isnum)
 				fprintf(fp,"%s=%d\n",config[i].var,
@@ -606,9 +592,9 @@ config_prepare(void)
 {
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(config); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		set_value(i,MOV_C2N);
-	panel_cfg.pd->top = panel_cfg.pd->curs = 0;
+	panel_cfg.pd->top = panel_cfg.pd->curs = panel_cfg.pd->min;
 	panel = panel_cfg.pd;
 	textline = 0;
 }
@@ -618,20 +604,17 @@ config_edit_num_prepare(void)
 {
 	static char prompt[CFGVAR_LEN + 48];
 	CNUM *pnum;
-	int param;
 
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	pnum = config[param].table;
-	
+	/* inherited panel = panel_cfg.pd */
+	pnum = config[panel_cfg.pd->curs].table;
 	textline = &line_tmp;
 	sprintf(prompt,"%s (range: %d - %d%s%s): ",
-	  config[param].var,pnum->min,pnum->max,
+	  config[panel_cfg.pd->curs].var,pnum->min,pnum->max,
 	  pnum->extra_val != 0 ? " or " : "",
 	  pnum->extra_val != 0 ? pnum->extra_val : "");
 	edit_setprompt(textline,prompt);
 
-	edit_nu_putstr(config_print_value(param));
-	edit_adjust();
+	edit_nu_putstr(config_print_value(panel_cfg.pd->curs));
 }
 
 void
@@ -639,29 +622,26 @@ config_edit_str_prepare(void)
 {
 	static char prompt[CFGVAR_LEN + 32];
 	CSTR *pstr;
-	int param;
-
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	pstr = config[param].table;
 
+	/* inherited panel = panel_cfg.pd */
+	pstr = config[panel_cfg.pd->curs].table;
 	textline = &line_tmp;
 	sprintf(prompt,"%s (%d chars max%s%s): ",
-	  config[param].var,CFGVALUE_LEN,
+	  config[panel_cfg.pd->curs].var,CFGVALUE_LEN,
 	  pstr->extra_val != 0 ? " or " : "",
 	  pstr->extra_val != 0 ? pstr->extra_val : "");
 	edit_setprompt(textline,prompt);
 
-	edit_nu_putstr(config_print_value(param));
-	edit_adjust();
+	edit_nu_putstr(config_print_value(panel_cfg.pd->curs));
 }
+
 void
 cx_config_num_enter(void)
 {
-	int nvalue, ilen, param;
+	int nvalue, ilen;
 	CNUM *pnum;
 
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	pnum = config[param].table;
+	pnum = config[panel_cfg.pd->curs].table;
 	if (pnum->extra_val != 0
 	  && strcmp(USTR(textline->line),pnum->extra_val) == 0) {
 		pnum->new = 0;
@@ -684,10 +664,8 @@ void
 cx_config_str_enter(void)
 {
 	CSTR *pstr;
-	int param;
 
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	pstr = config[param].table;
+	pstr = config[panel_cfg.pd->curs].table;
 	if (pstr->extra_val != 0
 	  && strcmp(USTR(textline->line),pstr->extra_val) == 0) {
 		*pstr->new = '\0';
@@ -706,25 +684,15 @@ cx_config_str_enter(void)
 void
 cx_config_default(void)
 {
-	int param;
-
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	if (param >= 0) {
-		set_value(param,MOV_I2N);
-		win_panel_opt();
-	}
+	set_value(panel_cfg.pd->curs,MOV_I2N);
+	win_panel_opt();
 }
 
 void
 cx_config_original(void)
 {
-	int param;
-
-	param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-	if (param >= 0) {
-		set_value(param,MOV_C2N);
-		win_panel_opt();
-	}
+	set_value(panel_cfg.pd->curs,MOV_C2N);
+	win_panel_opt();
 }
 
 /* detect what has changed */
@@ -735,7 +703,7 @@ config_changes(void)
 	CNUM *pnum;
 	CSTR *pstr;
 
-	for (i = 0; i < ARRAY_SIZE(config); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		if (config[i].isnum) {
 			pnum = config[i].table;
 			config[i].changed = pnum->new != pnum->current;
@@ -754,7 +722,7 @@ config_apply(void)
 	int i;
 	FLAG reread, sort, prompt;
 
-	for (i = 0; i < ARRAY_SIZE(config); i++)
+	for (i = 0; i < CFG_VARIABLES; i++)
 		if (config[i].changed)
 			set_value(i,MOV_N2C);
 
@@ -785,7 +753,8 @@ config_apply(void)
 	}
 	if (config[CFG_FMT_NUMBER].changed
 	  || config[CFG_FMT_TIME].changed
-	  || config[CFG_FMT_DATE].changed)
+	  || config[CFG_FMT_DATE].changed
+	  || config[CFG_KILOBYTE].changed)
 		reread = 1;
 	if (config[CFG_SHELLPROG].changed) {
 		exec_shell_reconfig();
@@ -826,43 +795,43 @@ config_apply(void)
 }
 
 void
-cx_config_edit(void)
+cx_config_enter(void)
 {
 	CNUM *pnum;
-	int param;
 
-	if (panel_cfg.pd->curs < panel_cfg.extra_lines) {
+	if (panel_cfg.pd->curs < 0) {
 		if (clex_data.admin) {
-			/* admin mode:  0 = CANCEL, 1 = SAVE */
-			if (panel_cfg.pd->curs == 1) {
+			/* admin mode:  -2 = EXIT, -1 = SAVE */
+			if (panel_cfg.pd->curs == -2)
+				next_mode = MODE_SPECIAL_QUIT;
+			else {
 				config_changes();
 				config_save();
 			}
-			next_mode = MODE_SPECIAL_QUIT;
 		}
 		else {
-			/* normal config: 0 = CANCEL, 1 = ACCEPT, 2 = SAVE */
-			if (panel_cfg.pd->curs >= 1) {
+			/* normal config: -3 = CANCEL, -2 = ACCEPT, -1 = SAVE */
+			if (panel_cfg.pd->curs >= -2) {
 				config_changes();
 				config_apply();
-				if (panel_cfg.pd->curs == 2)
+				if (panel_cfg.pd->curs == -1)
 					config_save();
 			}
 			next_mode = MODE_SPECIAL_RETURN;
 		}
-	} else {
-		param = panel_cfg.pd->curs - panel_cfg.extra_lines;
-		if (config[param].isnum) {
-			pnum = config[param].table;
-			if (pnum->desc[0]) {
-				if (++pnum->new > pnum->max)
-					pnum->new = pnum->min;
-			}
-			else
-				control_loop(MODE_CFG_EDIT_NUM);
+		return;
+	}
+
+	if (config[panel_cfg.pd->curs].isnum) {
+		pnum = config[panel_cfg.pd->curs].table;
+		if (pnum->desc[0]) {
+			if (++pnum->new > pnum->max)
+				pnum->new = pnum->min;
 		}
 		else
-			control_loop(MODE_CFG_EDIT_TXT);
-		win_panel_opt();
+			control_loop(MODE_CFG_EDIT_NUM);
 	}
+	else
+		control_loop(MODE_CFG_EDIT_TXT);
+	win_panel_opt();
 }
