@@ -47,7 +47,6 @@
 #include "edit.h"		/* edit_update() */
 #include "inout.h"		/* win_waitmsg() */
 #include "list.h"		/* stat2type() */
-#include "panel.h"		/* pan_adjust() */
 #include "sdstring.h"	/* SDSTR() */
 #include "userdata.h"	/* username_find() */
 #include "ustring.h"	/* USTR() */
@@ -188,7 +187,7 @@ completion_reconfig(void)
 
 	cc_max = cc_alloc = config_num(CFG_C_SIZE);
 	if (cc_max == 0)	/* C_PANEL_SIZE = AUTO */
-		cc_alloc = 60;	/* enough for every screen */
+		cc_alloc = 100;	/* enough for every screen */
 	CC_LIST = emalloc(cc_alloc * sizeof(COMPL_ENTRY));
 	for (i = 0; i < cc_alloc; i++)
 		SD_INIT(CC_LIST[i].str);
@@ -235,7 +234,7 @@ void
 compl_prepare(void)
 {
 	panel_compl.pd->cnt = compl.cnt;
-	panel_compl.pd->top = panel_compl.pd->curs = -1;
+	panel_compl.pd->top = panel_compl.pd->curs =  panel_compl.pd->min;
 	panel_compl.filenames = compl.filenames;
 	panel_compl.description = code2string(rq.type,1);
 
@@ -243,7 +242,7 @@ compl_prepare(void)
 	  config_num(CFG_COLLATION) ? qcmp1 : qcmp2);
 
 	panel = panel_compl.pd;
-	/* textline unchanged */
+	/* textline inherited from previous mode */
 }
 
 static void
@@ -463,11 +462,10 @@ complete_it(void)
 	/* C_PANEL_SIZE = AUTO */
 	if (config_num(CFG_C_SIZE) == 0) {
 		/*
-		 * substract 2 lines:
-		 * top line is reserved for -- (leave this panel) --
-		 * bottom line is left empty to show there is no need to scroll
+		 * substract extra lines and leave the bottom
+		 * line empty to show there is no need to scroll
 		 */
-		cc_max = display.panlines - 2;
+		cc_max = display.panlines + panel_compl.pd->min - 1;
 		LIMIT_MAX(cc_max,cc_alloc);
 	}
 
@@ -790,7 +788,6 @@ compl_dir(void)
 void
 cx_compl_complete(void)
 {
-	if (panel_compl.pd->curs >= 0)
-		insert_candidate(panel_compl.pd->curs);
+	insert_candidate(panel_compl.pd->curs);
 	next_mode = MODE_SPECIAL_RETURN;
 }
