@@ -27,6 +27,7 @@
 #include "clex.h"
 #include "control.h"
 
+#include "bookmarks.h"		/* bm_prepare() */
 #include "cfg.h"			/* config_prepare() */
 #include "completion.h"		/* compl_prepare() */
 #include "directory.h"		/* dir_main_prepare() */
@@ -72,9 +73,17 @@ typedef struct {
 	FLAG escp;			/* press escape key first */
 	int key;			/* if this key was pressed ... */
 	void (*fn)(void);	/* ... then this function is to be invoked */
+	int options;		/* options - see OPT_XXX below */
 } KEY_BINDING;
 
+/* extra panel lines - curs < 0 | regular panel lines - curs >= 0 */
+#define OPT_CURS	1	/* call the handler function only if
+						   the cursor is on a regular line */
+#define OPT_RET		2	/* like OPT_CURS but also leave the current mode
+						   if the cursor is on an extra line */
+
 #define CXM(X,M) void cx_mode_ ## X (void) { control_loop(MODE_ ## M); }
+static CXM(bm,BM)
 static CXM(cfg,CFG)
 static CXM(compare,COMPARE)
 static CXM(dir,DIR)
@@ -88,6 +97,7 @@ static CXM(sort,SORT)
 static CXM(deselect,DESELECT)
 
 #define CXT(X,M) void cx_trans_ ## X (void) { next_mode = MODE_ ## M; }
+static CXT(bm,BM)
 static CXT(quit,SPECIAL_QUIT)
 static CXT(return,SPECIAL_RETURN)
 
@@ -96,155 +106,177 @@ static void menu_prepare(void);
 static void cx_menu_pick(void);
 static void cx_menu_na(void);
 
+static KEY_BINDING tab_bm[] = {
+	{ 0,  CH_CTRL('M'),	cx_bm_enter,	0	},
+	{ 0,  0,			0,				0	}
+};
+
+static KEY_BINDING tab_bm_manager[] = {
+	{ 0,  CH_CTRL('C'),	cx_bm_manager_save,		0			},
+	{ 0,  CH_CTRL('M'),	cx_bm_manager_enter,	0			},
+	{ 0,  'd',			cx_bm_manager_down,		OPT_CURS	},
+	{ 0,  'n',			cx_bm_manager_new,		0			},
+	{ 0,  'u',			cx_bm_manager_up,		OPT_CURS	},
+	{ 0,  '\177',		cx_bm_manager_del,		OPT_CURS	},
+	{ 0,  KEY_DC,		cx_bm_manager_del,		OPT_CURS	},
+	{ 0,  0,			0,						0			}
+};
+
+static KEY_BINDING tab_bm_edit[] = {
+	{ 0,  CH_CTRL('M'),	cx_bm_edit_enter,	0	},
+	{ 0,  0,			0,					0	}
+};
+
 static KEY_BINDING tab_cfg[] = {
-	{ 0,  CH_CTRL('M'),	cx_config_edit			},
-	{ 0,  'd',			cx_config_default		},
-	{ 0,  'u',			cx_config_original		},
-	{ 0,  0,			0						}
+	{ 0,  CH_CTRL('M'),	cx_config_enter,	0			},
+	{ 0,  's',			cx_config_default,	OPT_CURS	},
+	{ 0,  'o',			cx_config_original,	OPT_CURS	},
+	{ 0,  0,			0,					0			}
 };
 
 static KEY_BINDING tab_cfg_edit_num[] = {
-	{ 0,  CH_CTRL('M'),	cx_config_num_enter	},
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('M'),	cx_config_num_enter,	0	},
+	{ 0,  0,			0,						0	}
 };
 
 static KEY_BINDING tab_cfg_edit_str[] = {
-	{ 0,  CH_CTRL('M'),	cx_config_str_enter	},
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('M'),	cx_config_str_enter,	0	},
+	{ 0,  0,			0,						0	}
 };
 
 static KEY_BINDING tab_common[] = {
-	{ 0,  CH_CTRL('C'),	cx_trans_return	},
-	{ 1,  'q',			cx_trans_quit	},
-	{ 1,  'v',			cx_version		},
-	{ 0,  CH_CTRL('G'),	cx_menu_na		},	/* catch bogus ctrl-G */
-	{ 0,  KEY_F(16),	cx_menu_na		},	/* catch bogus <MENU> */
-	{ 0,  KEY_F(1),		cx_mode_help	},
-	{ 0,  0,			0				}
+	{ 0,  CH_CTRL('C'),	cx_trans_return,	0	},
+	{ 1,  'q',			cx_trans_quit,		0	},
+	{ 1,  'v',			cx_version,			0	},
+	{ 0,  CH_CTRL('G'),	cx_menu_na,			0	},	/* catch bogus ctrl-G */
+	{ 0,  KEY_F(16),	cx_menu_na,			0	},	/* catch bogus <MENU> */
+	{ 0,  KEY_F(1),		cx_mode_help,		0	},
+	{ 0,  0,			0,					0	}
 };
 
 static KEY_BINDING tab_compare[] = {
-	{ 0,  CH_CTRL('M'),	cx_compare		},
-	{ 0,  '0',			cx_compare_0	},
-	{ 0,  '1',			cx_compare_1	},
-	{ 0,  '2',			cx_compare_2	},
-	{ 0,  '3',			cx_compare_3	},
-	{ 0,  '4',			cx_compare_4	},
-	{ 0,  0,			0				}
+	{ 0,  CH_CTRL('M'),	cx_compare,		0	},
+	{ 0,  '0',			cx_compare_0,	0	},
+	{ 0,  '1',			cx_compare_1,	0	},
+	{ 0,  '2',			cx_compare_2,	0	},
+	{ 0,  '3',			cx_compare_3,	0	},
+	{ 0,  '4',			cx_compare_4,	0	},
+	{ 0,  0,			0,				0	}
 };
 
 static KEY_BINDING tab_compl[] = {
-	{ 0,  CH_CTRL('I'),	cx_compl_complete	},
-	{ 0,  CH_CTRL('M'),	cx_compl_complete	},
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('I'),	cx_compl_complete,	OPT_RET	},
+	{ 0,  CH_CTRL('M'),	cx_compl_complete,	OPT_RET	},
+	{ 0,  0,			0,					0		}
 };
 
 static KEY_BINDING tab_dir[] = {
-	{ 0,  CH_CTRL('I'),	cx_dir_tab			},
-	{ 0,  CH_CTRL('M'),	cx_dir_enter		},
-	{ 0,  0,			0					}
+	{ 1,  'b',			cx_trans_bm,	0	},
+	{ 0,  CH_CTRL('I'),	cx_dir_tab,		0	},
+	{ 0,  CH_CTRL('M'),	cx_dir_enter,	0	},
+	{ 0,  0,			0,				0	}
 };
 
 static KEY_BINDING tab_edit[] = {
-	{ 0,  CH_CTRL('B'),	cx_edit_w_left		},
+	{ 0,  CH_CTRL('B'),	cx_edit_w_left,		0	},
 #ifdef KEY_SLEFT
-	{ 0,  KEY_SLEFT,	cx_edit_w_left		},
+	{ 0,  KEY_SLEFT,	cx_edit_w_left,		0	},
 #endif
-	{ 0,  CH_CTRL('D'),	cx_edit_w_del		},
-	{ 0,  CH_CTRL('H'),	cx_edit_backsp		},
-	{ 0,  CH_CTRL('U'),	cx_edit_kill		},
-	{ 0,  CH_CTRL('V'),	cx_edit_insert_spc	},
-	{ 0,  CH_CTRL('W'),	cx_edit_w_right		},
+	{ 0,  CH_CTRL('D'),	cx_edit_w_del,		0	},
+	{ 0,  CH_CTRL('H'),	cx_edit_backsp,		0	},
+	{ 0,  CH_CTRL('U'),	cx_edit_kill,		0	},
+	{ 0,  CH_CTRL('V'),	cx_edit_insert_spc,	0	},
+	{ 0,  CH_CTRL('W'),	cx_edit_w_right,	0	},
 #ifdef KEY_SRIGHT
-	{ 0,  KEY_SRIGHT,	cx_edit_w_right		},
+	{ 0,  KEY_SRIGHT,	cx_edit_w_right,	0	},
 #endif
-	{ 0,  CH_CTRL('Y'),	cx_edit_delend		},
-	{ 0,  CH_CTRL('Z'),	cx_undo				},
-	{ 1,  CH_CTRL('Z'),	cx_redo				},
-	{ 0,  '\177',		cx_edit_delchar		},
-	{ 0,  KEY_DC,		cx_edit_delchar		},
-	{ 0,  KEY_LEFT,		cx_edit_left		},
-	{ 0,  KEY_RIGHT,	cx_edit_right		},
-	{ 0,  KEY_HOME,		cx_edit_begin		},
+	{ 0,  CH_CTRL('Y'),	cx_edit_delend,		0	},
+	{ 0,  CH_CTRL('Z'),	cx_undo,			0	},
+	{ 1,  CH_CTRL('Z'),	cx_redo,			0	},
+	{ 0,  '\177',		cx_edit_delchar,	0	},
+	{ 0,  KEY_DC,		cx_edit_delchar,	0	},
+	{ 0,  KEY_LEFT,		cx_edit_left,		0	},
+	{ 0,  KEY_RIGHT,	cx_edit_right,		0	},
+	{ 0,  KEY_HOME,		cx_edit_begin,		0	},
 #ifdef KEY_END
-	{ 0,  KEY_END,		cx_edit_end			},
+	{ 0,  KEY_END,		cx_edit_end,		0	},
 #endif
-	{ 1,  KEY_UP,		cx_edit_up			},
-	{ 1,  KEY_DOWN,		cx_edit_down		},
-	{ 0,  0,			0					}
+	{ 1,  KEY_UP,		cx_edit_up,			0	},
+	{ 1,  KEY_DOWN,		cx_edit_down,		0	},
+	{ 0,  0,			0,					0	}
 };
 
 static KEY_BINDING tab_editcmd[] = {
-	{ 0,  CH_CTRL('E'),	cx_edit_paste_dir	},
-	{ 0,  CH_CTRL('G'),	cx_mode_mainmenu	},
-	{ 0,  KEY_F(16),	cx_mode_mainmenu	},
-	{ 0,  CH_CTRL('I'),	cx_files_tab		},
-	{ 1,  CH_CTRL('I'),	cx_edit_fullpath	},
-	{ 0,  CH_CTRL('M'),	cx_files_enter		},
-	{ 1,  CH_CTRL('M'),	cx_files_cd			},
-	{ 0,  CH_CTRL('K'),	cx_hist_complete	},
-	{ 0,  CH_CTRL('N'),	cx_hist_next		},
-	{ 0,  CH_CTRL('O'),	cx_edit_paste_link	},
-	{ 0,  CH_CTRL('P'),	cx_hist_prev		},
-	{ 0,  CH_CTRL('R'),	cx_files_reread		},
-	{ 1,  CH_CTRL('R'),	cx_files_reread_ug	},
-	{ 0,  CH_CTRL('T'),	cx_select_toggle	},
-	{ 0,  CH_CTRL('X'),	cx_files_exchange	},
-	{ 0,  KEY_IC,		cx_select_toggle	},
-	{ 0,  KEY_IL,		cx_select_toggle	},
-	{ 0,  KEY_F(2),		cx_edit_cmd_f2		},
-	{ 0,  KEY_F(3),		cx_edit_cmd_f3		},
-	{ 0,  KEY_F(4),		cx_edit_cmd_f4		},
-	{ 0,  KEY_F(5),		cx_edit_cmd_f5		},
-	{ 0,  KEY_F(6),		cx_edit_cmd_f6		},
-	{ 0,  KEY_F(7),		cx_edit_cmd_f7		},
-	{ 0,  KEY_F(8),		cx_edit_cmd_f8		},
-	{ 0,  KEY_F(9),		cx_edit_cmd_f9		},
-	{ 0,  KEY_F(10),	cx_edit_cmd_f10		},
-	{ 0,  KEY_F(11),	cx_edit_cmd_f11		},
-	{ 0,  KEY_F(12),	cx_edit_cmd_f12		},
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('E'),	cx_edit_paste_dir,	0			},
+	{ 0,  CH_CTRL('G'),	cx_mode_mainmenu,	0			},
+	{ 0,  KEY_F(16),	cx_mode_mainmenu,	0			},
+	{ 0,  CH_CTRL('I'),	cx_files_tab,		0			},
+	{ 1,  CH_CTRL('I'),	cx_edit_fullpath,	0			},
+	{ 0,  CH_CTRL('M'),	cx_files_enter,		0			},
+	{ 1,  CH_CTRL('M'),	cx_files_cd,		OPT_CURS	},
+	{ 0,  CH_CTRL('K'),	cx_hist_complete,	0			},
+	{ 0,  CH_CTRL('N'),	cx_hist_next,		0			},
+	{ 0,  CH_CTRL('O'),	cx_edit_paste_link,	OPT_CURS	},
+	{ 0,  CH_CTRL('P'),	cx_hist_prev,		0			},
+	{ 0,  CH_CTRL('R'),	cx_files_reread,	0			},
+	{ 1,  CH_CTRL('R'),	cx_files_reread_ug,	0			},
+	{ 0,  CH_CTRL('T'),	cx_select_toggle,	OPT_CURS	},
+	{ 0,  CH_CTRL('X'),	cx_files_exchange,	0			},
+	{ 0,  KEY_IC,		cx_select_toggle,	OPT_CURS	},
+	{ 0,  KEY_IL,		cx_select_toggle,	OPT_CURS	},
+	{ 0,  KEY_F(2),		cx_edit_cmd_f2,		0			},
+	{ 0,  KEY_F(3),		cx_edit_cmd_f3,		0			},
+	{ 0,  KEY_F(4),		cx_edit_cmd_f4,		0			},
+	{ 0,  KEY_F(5),		cx_edit_cmd_f5,		0			},
+	{ 0,  KEY_F(6),		cx_edit_cmd_f6,		0			},
+	{ 0,  KEY_F(7),		cx_edit_cmd_f7,		0			},
+	{ 0,  KEY_F(8),		cx_edit_cmd_f8,		0			},
+	{ 0,  KEY_F(9),		cx_edit_cmd_f9,		0			},
+	{ 0,  KEY_F(10),	cx_edit_cmd_f10,	0			},
+	{ 0,  KEY_F(11),	cx_edit_cmd_f11,	0			},
+	{ 0,  KEY_F(12),	cx_edit_cmd_f12,	0			},
+	{ 0,  0,			0,					0			}
 };
 
 static KEY_BINDING tab_help[] = {
-	{ 0,  CH_CTRL('H'),	cx_help_back		},
-	{ 0,  CH_CTRL('M'),	cx_help_link		},
-	{ 0,  KEY_LEFT,		cx_help_back		},
-	{ 0,  KEY_RIGHT,	cx_help_link		},
-	{ 0,  KEY_F(1),		cx_help_contents	},	/* avoid F1 recursion */
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('H'),	cx_help_back,		0	},
+	{ 0,  CH_CTRL('M'),	cx_help_link,		0	},
+	{ 0,  KEY_LEFT,		cx_help_back,		0	},
+	{ 0,  KEY_RIGHT,	cx_help_link,		0	},
+	{ 0,  KEY_F(1),		cx_help_contents,	0	},	/* avoid F1 recursion */
+	{ 0,  0,			0,					0	}
 };
 
 static KEY_BINDING tab_hist[] = {
-	{ 0,  '\177',		cx_hist_del		},
-	{ 0,  KEY_DC,		cx_hist_del		},
-	{ 0,  CH_CTRL('I'),	cx_hist_paste	},
-	{ 0,  CH_CTRL('M'),	cx_hist_enter	},
-	{ 0,  CH_CTRL('N'),	cx_pan_up		},	/* redefine history next */
-	{ 0,  CH_CTRL('P'),	cx_pan_down		},	/* redefine history prev */
+	{ 0,  '\177',		cx_hist_del,	OPT_CURS	},
+	{ 0,  KEY_DC,		cx_hist_del,	OPT_CURS	},
+	{ 0,  CH_CTRL('I'),	cx_hist_paste,	OPT_CURS	},
+	{ 0,  CH_CTRL('M'),	cx_hist_enter,	0			},
+	{ 0,  CH_CTRL('N'),	cx_pan_up,		0			},	/* redefine history next */
+	{ 0,  CH_CTRL('P'),	cx_pan_down,	0			},	/* redefine history prev */
 	{ 0,  0,			0				}
 };
 
 /* pseudo-table returned by do_action() */
 static KEY_BINDING tab_insertchar[] = {
-	{ 0, 0,				0				}
+	{ 0, 0,	0, 0	}
 };
 
 static KEY_BINDING tab_locate[] = {
-	{ 0,  CH_CTRL('I'),	cx_locate_tab	},
-	{ 0,  CH_CTRL('M'),	cx_locate_go	},
-	{ 0,  KEY_F(2),		cx_edit_cmd_f2	},
-	{ 0,  KEY_F(3),		cx_edit_cmd_f3	},
-	{ 0,  KEY_F(4),		cx_edit_cmd_f4	},
-	{ 0,  KEY_F(5),		cx_edit_cmd_f5	},
-	{ 0,  KEY_F(6),		cx_edit_cmd_f6	},
-	{ 0,  KEY_F(7),		cx_edit_cmd_f7	},
-	{ 0,  KEY_F(8),		cx_edit_cmd_f8	},
-	{ 0,  KEY_F(9),		cx_edit_cmd_f9	},
-	{ 0,  KEY_F(10),	cx_edit_cmd_f10	},
-	{ 0,  KEY_F(11),	cx_edit_cmd_f11	},
-	{ 0,  KEY_F(12),	cx_edit_cmd_f12	},
-	{ 0,  0,			0				}
+	{ 0,  CH_CTRL('I'),	cx_locate_tab,		OPT_CURS	},
+	{ 0,  CH_CTRL('M'),	cx_locate_go,		OPT_RET		},
+	{ 0,  KEY_F(2),		cx_edit_cmd_f2,		0			},
+	{ 0,  KEY_F(3),		cx_edit_cmd_f3,		0			},
+	{ 0,  KEY_F(4),		cx_edit_cmd_f4,		0			},
+	{ 0,  KEY_F(5),		cx_edit_cmd_f5,		0			},
+	{ 0,  KEY_F(6),		cx_edit_cmd_f6,		0			},
+	{ 0,  KEY_F(7),		cx_edit_cmd_f7,		0			},
+	{ 0,  KEY_F(8),		cx_edit_cmd_f8,		0			},
+	{ 0,  KEY_F(9),		cx_edit_cmd_f9,		0			},
+	{ 0,  KEY_F(10),	cx_edit_cmd_f10,	0			},
+	{ 0,  KEY_F(11),	cx_edit_cmd_f11,	0			},
+	{ 0,  KEY_F(12),	cx_edit_cmd_f12,	0			},
+	{ 0,  0,			0,					0			}
 };
 
 static KEY_BINDING tab_mainmenu[] = {
@@ -252,79 +284,88 @@ static KEY_BINDING tab_mainmenu[] = {
  * these lines correspond with the main menu panel,
  * if you change this, you must update initialization
  * in start.c and descriptions in inout.c
+ *
+ * Warning: OPT_CURS or OPT_RET options will check
+ * panel_mainmenu, this is probably not what you want !
  */
-	{ 0,  0,			cx_mode_help		},
+	{ 0,  0,			cx_mode_help,		0	},
 	/*
 	 * no key for cx_mode_help, note the difference:
 	 * <F1> in tab_common:    main menu -> help -> main menu
 	 * help in tab_mainmenu:  main menu -> help -> file panel
 	 */
-	{ 1,  'd',			cx_mode_dir			},
-	{ 1,  '/',			cx_files_cd_root	},
-	{ 1,  '.',			cx_files_cd_parent	},
-	{ 1,  '~',			cx_files_cd_home	},
-	{ 1,  'h',			cx_mode_history		},
-	{ 1,  's',			cx_mode_sort		},
-	{ 0,  CH_CTRL('R'),	cx_files_reread		},
-	{ 0,  CH_CTRL('F'),	cx_mode_s_locate	},
-	{ 1,  CH_CTRL('F'),	cx_mode_p_locate	},
-	{ 1,  '=',			cx_mode_compare		},
+	{ 1,  'd',			cx_mode_dir,		0	},
+	{ 1,  '/',			cx_files_cd_root,	0	},
+	{ 1,  '.',			cx_files_cd_parent,	0	},
+	{ 1,  '~',			cx_files_cd_home,	0	},
+	{ 1,  'b',			cx_mode_bm,			0	},
+	{ 1,  'h',			cx_mode_history,	0	},
+	{ 1,  's',			cx_mode_sort,		0	},
+	{ 0,  CH_CTRL('R'),	cx_files_reread,	0	},
+	{ 0,  CH_CTRL('F'),	cx_mode_s_locate,	0	},
+	{ 1,  CH_CTRL('F'),	cx_mode_p_locate,	0	},
+	{ 1,  '=',			cx_mode_compare,	0	},
 	/*
 	 * No keys for next 2 functions because this table is valid
      * both for the file panel and the main menu. Keys specific
      * for the main menu are listed in the tab_mainmenu2[] below.
 	 */
-	{ 0,  0,			cx_select_allfiles	},		/* no key */
-	{ 0,  0,			cx_select_nofiles	},		/* no key */
-	{ 1,  '+',			cx_mode_select		},
-	{ 1,  '-',			cx_mode_deselect	},
-	{ 1,  '*',			cx_select_invert	},
-	{ 1,  'c',			cx_mode_cfg			},
-	{ 1,  'v',			cx_version			},
-	{ 0,  0,			cx_trans_quit		},	/* no key necessary */
-	{ 0,  0,			0					}
+	{ 0,  0,			cx_select_allfiles,	0	},		/* no key */
+	{ 0,  0,			cx_select_nofiles,	0	},		/* no key */
+	{ 1,  '+',			cx_mode_select,		0	},
+	{ 1,  '-',			cx_mode_deselect,	0	},
+	{ 1,  '*',			cx_select_invert,	0	},
+	{ 1,  'c',			cx_mode_cfg,		0	},
+	{ 1,  'v',			cx_version,			0	},
+	{ 0,  0,			cx_trans_quit,		0	},	/* no key necessary */
+	{ 0,  0,			0,					0	}
 };
 
 static KEY_BINDING tab_mainmenu2[] = {
-	{ 0,  CH_CTRL('M'),	cx_menu_pick		},
-	{ 0,  '+',			cx_select_allfiles	},
-	{ 0,  '-',			cx_select_nofiles	},
-	{ 0,  0,			0					}
+	{ 0,  CH_CTRL('M'),	cx_menu_pick,		OPT_RET	},
+	{ 0,  '+',			cx_select_allfiles,	0		},
+	{ 0,  '-',			cx_select_nofiles,	0		},
+	{ 0,  0,			0,					0		}
 };
 
 static KEY_BINDING tab_panel[] = {
-	{ 0,  KEY_UP,		cx_pan_up		},
-	{ 0,  KEY_DOWN,		cx_pan_down		},
-	{ 0,  KEY_PPAGE,	cx_pan_pgup		},
-	{ 0,  KEY_NPAGE,	cx_pan_pgdown	},
-	{ 1,  KEY_HOME,		cx_pan_home		},
+	{ 0,  KEY_UP,		cx_pan_up,		0	},
+	{ 0,  KEY_DOWN,		cx_pan_down,	0	},
+	{ 0,  KEY_PPAGE,	cx_pan_pgup,	0	},
+	{ 0,  KEY_NPAGE,	cx_pan_pgdown,	0	},
+	{ 1,  KEY_HOME,		cx_pan_home,	0	},
 #ifdef KEY_END
-	{ 1,  KEY_END,		cx_pan_end		},
+	{ 1,  KEY_END,		cx_pan_end,		0	},
 #endif
-	{ 0,  0,			0				}
+	{ 0,  0,			0,				0	}
 };
 
 static KEY_BINDING tab_select[] = {
-	{ 0,  CH_CTRL('M'),	cx_select_files	},
-	{ 0,  0,			0				}
+	{ 0,  CH_CTRL('M'),	cx_select_files,	0	},
+	{ 0,  0,			0,					0	}
 };
 
 static KEY_BINDING tab_sort[] = {
-	{ 0,  ' ',			cx_sort_set		},
-	{ 0,  CH_CTRL('M'),	cx_sort_set		},
-	{ 0,  0,			0				}
+	{ 0,  CH_CTRL('M'),	cx_sort_set,	OPT_RET		},
+	{ 0,  0,			0,				0			}
 };
 
 typedef struct {
 	CODE mode;
 	void (*prepare_fn)(void);
-	KEY_BINDING *table[6];	/* up to 6 tables terminated with NULL;
+	KEY_BINDING *table[6];	/* up to 5 tables terminated with NULL;
 							   order is sometimes important, only
 							   the first KEY_BINDING for a given key
 							   is executed */
 } MODE_DEFINITION;
 
 static MODE_DEFINITION mode_definition[] = {
+	{ MODE_BM, bm_prepare,
+		{ tab_bm,tab_panel,tab_common,0 } },
+	{ MODE_BM_MANAGER, bm_manager_prepare,
+		{ tab_bm_manager,tab_panel,tab_common,0 } },
+	{ MODE_BM_EDIT, bm_edit_prepare,
+		{ tab_bm_edit,tab_edit,tab_common,0 } },
 	{ MODE_CFG_EDIT_NUM, config_edit_num_prepare,
 		{ tab_cfg_edit_num,tab_edit,tab_common,0 } },
 	{ MODE_CFG_EDIT_TXT, config_edit_str_prepare,
@@ -395,6 +436,22 @@ get_modedef(int mode)
 	return 0;
 }
 
+/* return value: true if it is OK to call the handler function */
+static int
+fn_options(int opt)
+{
+	if (opt == 0)
+		return 1;
+
+	if (opt == OPT_RET && panel->curs < 0) {
+		next_mode = MODE_SPECIAL_RETURN;
+		return 0; 
+	}
+
+	/* OPT_CURS or OPT_RET */
+	return panel->cnt > 0 && panel->curs >= 0;
+}
+
 static KEY_BINDING *
 do_action(int key, KEY_BINDING **tables)
 {
@@ -425,13 +482,15 @@ do_action(int key, KEY_BINDING **tables)
 					}
 				}
 				else if (esc || !tab[i].escp) {
-					(*tab[i].fn)();
+					if (fn_options(tab[i].options))
+						(*tab[i].fn)();
 					return tab;
 				}
 			}
 	}
 	if (noesc_tab) {
-		(*noesc_tab[noesc_idx].fn)();
+		if (fn_options(noesc_tab[noesc_idx].options))
+			(*noesc_tab[noesc_idx].fn)();
 		return noesc_tab;
 	}
 
@@ -473,11 +532,13 @@ control_loop(int mode)
 		win_heading();
 		if (panel != clex_mode->panel) {
 			clex_mode->panel = panel;
+			pan_adjust(panel);
 			win_panel();
 		}
 		if (textline != clex_mode->textline) {
 			undo_reset();
 			clex_mode->textline = textline;
+			edit_adjust();
 			win_edit();
 		}
 
@@ -497,10 +558,6 @@ control_loop(int mode)
 
 			/* some special handling not implemented with tables */
 			switch (clex_mode->mode) {
-			case MODE_MAINMENU:
-				if (kb_tab == tab_mainmenu || kb_tab == tab_mainmenu2)
-					next_mode = MODE_SPECIAL_RETURN;
-				break;
 			case MODE_COMPL:
 				if (kb_tab == tab_edit || kb_tab == tab_insertchar)
 					next_mode = MODE_SPECIAL_RETURN;
@@ -513,13 +570,16 @@ control_loop(int mode)
 					win_panel_opt();
 				}
 				break;
+			case MODE_MAINMENU:
+				if (kb_tab == tab_mainmenu || kb_tab == tab_mainmenu2)
+					next_mode = MODE_SPECIAL_RETURN;
+				break;
 			case MODE_P_LOCATE:
 			case MODE_S_LOCATE:
 				if (textline == &line_cmd)
 					next_mode = MODE_SPECIAL_RETURN;
 				else
 					locate_update();
-				break;
 			}
 			if (next_mode)
 				break;
@@ -553,15 +613,13 @@ menu_prepare(void)
 {
 	/* leave cursor position unchanged */
 	panel = panel_mainmenu.pd;
-	pan_adjust(panel);
 	textline = 0;
 }
 
 static void
 cx_menu_pick(void)
 {
-	if (panel_mainmenu.pd->curs >= 0)
-		(*tab_mainmenu[panel_mainmenu.pd->curs].fn)();
+	(*tab_mainmenu[panel_mainmenu.pd->curs].fn)();
 	if (next_mode == 0)
 		next_mode = MODE_SPECIAL_RETURN;
 }
