@@ -385,8 +385,6 @@ cx_edit_paste_link(void)
 {
 	FILE_ENTRY *pfe;
 
-	if (ppanel_file->pd->cnt == 0)
-		return;
 	pfe = ppanel_file->files[ppanel_file->pd->curs];
 	if (!pfe->symlink)
 		win_remark("not a symbolic link");
@@ -400,16 +398,11 @@ cx_edit_paste_link(void)
 int
 edit_isspecial(int ch)
 {
-	static const char *cfg_quote = 0;
-
-	if (cfg_quote == 0)
-		cfg_quote = config_str(CFG_QUOTE);
-
 	/* built-in metacharacters (let's play it safe) */
 	if (strchr("\t ()<>[]{}!#$&\\|?*:;\'\"`~",ch))
 		return 1;
 	/* additional special characters to be quoted */
-	if (*cfg_quote && strchr(cfg_quote,ch))
+	if (*config_str(CFG_QUOTE) && strchr(config_str(CFG_QUOTE),ch))
 		return 1;
 
 	return 0;
