@@ -43,8 +43,8 @@ void
 files_initialize(void)
 {
 	const char *dir2;
-	static PANEL_DESC panel_desc_1 = { 0,0,0,0,PANEL_TYPE_FILE,0,1 };
-	static PANEL_DESC panel_desc_2 = { 0,0,0,0,PANEL_TYPE_FILE,0,1 };
+	static PANEL_DESC panel_desc_1 = { 0,0,0,0,PANEL_TYPE_FILE,0 };
+	static PANEL_DESC panel_desc_2 = { 0,0,0,0,PANEL_TYPE_FILE,0 };
 	static PANEL_FILE panel_f1 =
 		{ &panel_desc_1, { 0,0 }, 0,0,1,0,0 };
 	static PANEL_FILE panel_f2 =
@@ -103,8 +103,8 @@ find_valid_cwd(void)
 {
 	char *dir, *p;
 
-	win_warning("CHANGE DIR: panel's directory is not accessible "
-	  "and will be changed");
+	win_warning("CHANGE DIR: cannot access panel's directory, "
+	  "it will be changed.");
 
 	ppanel_file->pd->cnt = ppanel_file->selected = 0;
 
@@ -208,9 +208,6 @@ cx_files_cd(void)
 {
 	FILE_ENTRY *pfe;
 
-	if (ppanel_file->pd->cnt == 0)
-		return;
-
 	pfe = ppanel_file->files[ppanel_file->pd->curs];
 	if (IS_FT_DIR(pfe->file_type)) {
 		if (changedir(SDSTR(pfe->file)) == 0) {
