@@ -18,7 +18,7 @@
  * - helpfile is a plain text file
  * - comments beginning with '#' on the first column are ignored
  * - please no TABs, always use SPACEs
- * - maximal width is 60 columns
+ * - maximal width is 60 columns (MIN_COLS - 4)
  *
  * help file:
  *    @V=VERSION  <-- program version
@@ -41,14 +41,11 @@
 #include <config.h>
 
 #include <sys/types.h>	/* clex.h */
-#include <sys/stat.h>	/* fstat() */
 #include <errno.h>		/* errno */
-#include <fcntl.h>		/* open() */
 #include <stdarg.h>		/* va_list */
 #include <stdlib.h>		/* free() */
 #include <stdio.h>		/* vprintf() */
 #include <string.h>		/* strcmp() */
-#include <unistd.h>		/* close() */
 
 #include "clex.h"
 #include "help.h"
@@ -61,12 +58,12 @@
 
 #define HELP_HISTORY_ALLOC	16
 
-/* limits to prevent resource misuse */
+/* limits to protect resources */
 #define HELP_PAGES_LIMIT	60			/* pages max */
-#define HELP_FILESIZE_LIMIT	75000L		/* bytes max (type size_t) */
+#define HELP_FILESIZE_LIMIT	75000		/* bytes max */
 
 /* help sources: external(optional) and built-in */
-static char *external_help = 0;	/* contents of the optional help file */
+static char *ext_help = 0;	/* contents of the optional help file */
 static HELP_ENTRY built_in_help[] = {
 #	include "help.inc"
 	{ 0, 0 }
@@ -75,7 +72,7 @@ static HELP_ENTRY built_in_help[] = {
 /* parsed help */
 static HELP_ENTRY *helpline = 0;	/* help split into lines */
 static struct {
-	const char *name;			/* page name (useful for links) */
+	const char *name;			/* page name (for links) */
 	const char *heading;		/* heading to be displayed */
 	int firstline;				/* first line = 'helpline[]' index */
 	int size;					/* total number of lines in this page */
@@ -135,46 +132,30 @@ help_error(const char *format, ...)
 static int
 read_external_help(const char *filename)
 {
-	int fd, line;
-	char *ptr, *saveptr, *aux;
-	struct stat stbuf;
+	int error, line;
+	char *ptr, *eot;
 	size_t filesize;
-	ssize_t rd;
 
-	if ( (fd = open(filename,O_RDONLY)) < 0) {
-		help_error("Cannot open the file (%s)\n",strerror(errno));
+	filesize = HELP_FILESIZE_LIMIT;
+	ext_help = read_file(filename,&filesize,&error);
+	if (ext_help == 0) {
+		switch (error) {
+		case 1:
+			help_error("Cannot open the file (%s)\n",strerror(errno));
+			break;
+		case 2:
+			help_error("File is too big, limit is "
+			  STR(HELP_FILESIZE_LIMIT) " bytes\n");
+			break;
+		case 3:
+			help_error("File read error (%s)\n",strerror(errno));
+		}
 		return -1;
 	}
 
-	/* check file size */
-	fstat(fd,&stbuf);	/* cannot fail with valid descriptor */
-	if (stbuf.st_size == 0) {
-		close(fd);
-		help_error("File is empty\n");
-		return -1;
-	}
-	if (stbuf.st_size > HELP_FILESIZE_LIMIT) {
-		close(fd);
-		help_error("File is too big, limit is %d bytes\n",
-		  HELP_FILESIZE_LIMIT);
-		return -1;
-	}
-
-	/* read into memory */
-	filesize = stbuf.st_size;
-	external_help = emalloc(filesize + 1);
-	rd = read_fd(fd,external_help,filesize);
-	close(fd);
-	if (rd == -1) {
-		help_error("File read error (%s)\n",strerror(errno));
-		return -1;
-	}
-	if (external_help[filesize - 1] != '\n')
-		external_help[filesize++] = '\n';
-
 	/* count lines + add 1 extra */
-	for (line = 1, ptr = external_help;
-	  ptr < external_help + filesize; ptr++)
+	line = 1;
+	for (ptr = ext_help; ptr < ext_help + filesize; ptr++)
 		if (*ptr == '\t' || *ptr == '\r')
 			*ptr = ' ';
 		else if (*ptr == '\n' || *ptr == '\0') {
@@ -184,23 +165,20 @@ read_external_help(const char *filename)
 	helpline = emalloc(line * sizeof(HELP_ENTRY));
 
 	line = 0;
-	for (ptr = external_help; ptr < external_help + filesize; ) {
+	for (ptr = ext_help; ptr < ext_help + filesize; ) {
 		if (*ptr != '#') {
 			helpline[line].txt = ptr;
 			helpline[line].aux = 0;
 			/* split XXXX @@=YYYY, remove trailing spaces after XXXX */
-			for (aux = ptr; *aux; aux++) {
-				if (aux[0] == '@' && aux[1] == '@' && aux[2] == '=') {
-					helpline[line].aux = saveptr = aux + 3;
-					for (*aux = ' '; *aux == ' '; aux--) {
-						*aux = '\0';
-						if (aux == ptr)
-							break;
-					}
-					/* skip over new '\0's */
-					ptr = saveptr;
+			for (eot = ptr; *ptr; ptr++) {
+				if (ptr[0] == '@' && ptr[1] == '@' && ptr[2] == '=') {
+					*ptr = *eot = '\0';
+					ptr += 3;
+					helpline[line].aux = ptr;
 					break;
 				}
+				else if (*ptr != ' ')
+					eot = ptr + 1;
 			}
 			line++;
 		}
@@ -228,9 +206,9 @@ parse_help(const char *filename)
 	help_error_init(filename);
 
 	/* erase previous help */
-	if (external_help) {
-		free(external_help);
-		external_help = 0;
+	if (ext_help) {
+		free(ext_help);
+		ext_help = 0;
 	}
 	if (helpline) {
 		if (helpline != built_in_help)
@@ -261,8 +239,8 @@ parse_help(const char *filename)
 			/* page begin */
 			ptr += 3;
 			if (pagecnt == HELP_PAGES_LIMIT) {
-				help_error("Too many help pages, limit is %d\n",
-				  HELP_PAGES_LIMIT);
+				help_error("Too many help pages, limit is "
+				  STR(HELP_PAGES_LIMIT) "\n");
 				return -1;
 			}
 			if (TSET(pagestart))
@@ -328,12 +306,9 @@ parse_help(const char *filename)
 void
 help_reconfig(void)
 {
-	const char *filename;
-
-	filename = config_str(CFG_HELPFILE);
-	if (*filename) {
+	if (*config_str(CFG_HELPFILE)) {
 		/* external help */
-		if (parse_help(filename) == 0)
+		if (parse_help(config_str(CFG_HELPFILE)) == 0)
 			return;
 		help_error("Using the built-in help instead\n");
 	}
@@ -350,7 +325,7 @@ set_page(int pg, const char *link)
 	int i;
 	const char *aux;
 
-	panel_help.pd->top = panel_help.pd->curs = 0;
+	panel_help.pd->top = panel_help.pd->curs = panel_help.pd->min;
 	panel_help.pd->cnt = helppage[pg].size;
 	panel_help.pagenum = pg;
 	panel_help.heading = helppage[pg].heading;
@@ -404,6 +379,11 @@ help_prepare(void)
 
 	/* CLEX help is context sensitive */
 	switch (get_previous_mode()) {
+	case MODE_BM:
+	case MODE_BM_MANAGER:
+	case MODE_BM_EDIT:
+		link = "bookmarks";
+		break;
 	case MODE_CFG:
 	case MODE_CFG_EDIT_NUM:
 	case MODE_CFG_EDIT_TXT:
