@@ -36,7 +36,6 @@
 
 #include "cfg.h"		/* config_num() */
 #include "control.h"	/* get_current_mode() */
-#include "directory.h"	/* dir_split_len() */
 #include "edit.h"		/* edit_adjust() */
 #include "panel.h"		/* pan_adjust() */
 #include "sdstring.h"	/* SDSTR() */
@@ -91,7 +90,6 @@ static FLAG pos_panel = 0;	/* --< CURSOR/TOTAL >-- normal info */
 						 */ 
 static int prevkey = 0;			/* previously pressed key */
 static chtype attrr, attrb;		/* reverse and bold or substitutes */
-static void (*draw_line)(int);	/* function drawing panel lines */
 static int framechar;			/* panel frame character
 								  (note that ACS_HLINE is an int) */
 
@@ -128,12 +126,13 @@ screen_draw_all(void)
 			break;
 		/* window too small */
 		printw("SCREEN: this window %d x %d is too small, "
-		  "required is %d x %d or larger; press ctrl-C to "
+		  "required is " STR(MIN_LINES) " x " STR(MIN_COLS)
+		  " or larger; press ctrl-C to "
 		  "exit the CLEX or (if possible) enlarge the window"
 #ifndef KEY_RESIZE
 		  " and press any other key to continue"
 #endif
-		  ". ",y,x,MIN_LINES,MIN_COLS);
+		  ". ",y,x);
 		refresh();
 		if (getch() == CH_CTRL('C'))
 			err_exit("Window is too small");
@@ -388,7 +387,7 @@ win_layout_reconfig(void)
 	FLAG fld;
 	char ch, *pch;
 
-	us_copy(&layout,config_str(CFG_LAYOUT));
+	us_copy(&layout,config_layout);
 
 	/* split layout to panel fields and line fields */
 	layout_panel = USTR(layout);
@@ -470,6 +469,11 @@ win_position(void)
 		return;
 	}
 
+	if (panel->curs < 0) {
+		print_position("",0,0);
+		return;
+	}
+
 	if (panel->type == PANEL_TYPE_FILE && ppanel_file->selected)
 		len = sprintf(buffer,"< [%d] %d/%d >",
 		  ppanel_file->selected,panel->curs + 1,panel->cnt);
@@ -599,13 +603,23 @@ win_heading(void)
 	addch(' ');
 	len = 1;
 	switch (mode) {
+	case MODE_BM:
+		msg = "CHANGE WORKING DIRECTORY > BOOKMARKS";
+		break;
+	case MODE_BM_MANAGER:
+		msg = "BOOKMARK MANAGER   "
+		  "[ U/D = up/down, N = new, <del> = delete, <enter> = edit ]";
+		break;
+	case MODE_BM_EDIT:
+		msg = "BOOKMARK MANAGER > EDIT";
+		break;
 	case MODE_CFG:
 		msg = "CONFIGURATION    "
-		  "[ <enter> = change, U = undo, D = default ]";
+		  "[ <enter> = change, O = original, S = standard ]";
 		break;
 	case MODE_CFG_EDIT_NUM:
 	case MODE_CFG_EDIT_TXT:
-		msg = "CONFIGURATION > NEW VALUE";
+		msg = "CONFIGURATION > EDIT";
 		break;
 	case MODE_COMPARE:
 		msg = "DIRECTORY COMPARE";
@@ -641,7 +655,7 @@ win_heading(void)
 	case MODE_DESELECT:
 		msg = "DESELECT FILES";
 		if (mode == MODE_SELECT)
-			msg +=2;	/* DESELECT... --> SELECT ... */
+			msg += 2;	/* DESELECT... --> SELECT ... */
 		break;
 	case MODE_SORT:
 		msg = "SORT ORDER";
@@ -721,7 +735,7 @@ print_fields(FILE_ENTRY *pfe, int width, const char *fields)
 				fw = FE_MODE_STR - 1;
 				txt = pfe->mode_str;
 				break;
-			case 'M':	/* file mode (alternative format*/
+			case 'M':	/* file mode (alternative format) */
 				fw = FE_MODE_STR - 1;
 				txt = pfe->normal_mode ? "" : pfe->mode_str;
 				break;
@@ -796,21 +810,58 @@ print_fields(FILE_ENTRY *pfe, int width, const char *fields)
 	return width;
 }
 
+static void
+win_info_extra_line(int ln)
+{
+	const char *msg;
+	static const char *infoline_cfg[] = {
+		"  No configuration change",
+		"  Use the new configuration in this session",
+		"  Save the configuration to ~/.clexrc"
+	},
+	*infoline_cfg_admin[] = {
+		"  Exit program",
+		"  Save the new system-wide defaults to " CONFIG_FILE
+	};
+
+	msg = 0;
+	switch (panel->type) {
+	case PANEL_TYPE_CFG:
+		msg = infoline_cfg[ln];
+		break;
+	case PANEL_TYPE_CFG_ADMIN:
+		msg = infoline_cfg_admin[ln];
+		break;
+	case PANEL_TYPE_BM:
+		if (ln == 2) {
+			addstr("  working directory is "); /* 23 */
+			putstr_trunc(USTR(ppanel_file->dir),display.scrcols - 23,0);
+			return;
+		}
+		break;
+	}
+	if (msg)
+		putstr_trunc(msg,display.scrcols,0);
+	else
+		clrtoeol();
+}
+
+static void
+pfe_info(FILE_ENTRY *pfe)
+{
+	if (pfe->file_type == FT_NA)
+		addstr("no status information available");
+	else if (print_fields(pfe,display.scrcols,layout_line) == 0)
+		return;
+	clrtoeol();
+}
+
 /* information line */
 static void
 win_info(void)
 {
-	static const char *cfg_normal_infoline[] = {
-		"No configuration change",
-		"Use the new configuration in this session",
-		"Save the configuration to ~/.clexrc"
-	}, *cfg_admin_infoline[] = {
-		"No configuration change",
-		"Save the new system-wide defaults to " CONFIG_FILE
-	};
 	int width;
 	const char *msg;
-	FILE_ENTRY *pfe;
 
 	move(display.panlines + 3,0);
 
@@ -822,7 +873,8 @@ win_info(void)
 		  putstr_trunc(info_warnmsg,display.scrcols - 15,OPT_NOPAD);
 		attrset(A_NORMAL);
 		addstr(" Press any key.");		/* 15 */
-		BLANK(display.scrcols - width - 15);
+		if (display.scrcols - width > 15)
+			clrtoeol();
 		return;
 	}
 
@@ -832,47 +884,46 @@ win_info(void)
 		width = putstr_trunc(info_remark,display.scrcols - 6,OPT_NOPAD);
 		addstr(" --");					/* 3 */
 		attrset(A_NORMAL);
-		BLANK(display.scrcols - width - 6);
+		if (display.scrcols - width > 6)
+			clrtoeol();
 		return;
 	}
 
-	if (panel->cnt == 0 || panel->curs < 0 || !panel->infoline) {
+	if (panel->curs < 0 && panel->min < 0) {
+		/* extra panel lines */
+		win_info_extra_line(panel->curs - panel->min);
+		return;
+	}
+
+	if (panel->cnt == 0) {
 		clrtoeol();
 		return;
 	}
 
-	if (panel->type == PANEL_TYPE_CFG) {
-		addstr("  ");					/* 2 */
-		if (panel->curs < panel_cfg.extra_lines)
-			msg = (clex_data.admin ? cfg_admin_infoline
-			  : cfg_normal_infoline)[panel->curs];
-		else
-			msg = panel_cfg.config[panel->curs
-			  - panel_cfg.extra_lines].help;
-		putstr_trunc(msg,display.scrcols - 2,0);
-		return;
-	}
-
-	if (panel->type == PANEL_TYPE_COMPL) {
-		if ( (msg = panel_compl.candidate[panel->curs].aux) ) {
-			addstr("  ");				/* 2 */
+	switch (panel->type) {
+		case PANEL_TYPE_CFG:
+		case PANEL_TYPE_CFG_ADMIN:
+			addstr("  ");					/* 2 */
+			msg = panel_cfg.config[panel->curs].help;
 			putstr_trunc(msg,display.scrcols - 2,0);
-			return;
-		}
-		clrtoeol();
-		return;
+			break;
+		case PANEL_TYPE_COMPL:
+			if ( (msg = panel_compl.candidate[panel->curs].aux) ) {
+				addstr("  location: ");		/* 12 */
+				putstr_trunc(msg,display.scrcols - 12,0);
+			}
+			else
+				clrtoeol();
+			break;
+		case PANEL_TYPE_LOCATE:
+			pfe_info(panel_locate.files[panel->curs]);
+			break;
+		case PANEL_TYPE_FILE:
+			pfe_info(ppanel_file->files[panel->curs]);
+			break;
+		default:
+			clrtoeol();
 	}
-
-	pfe = panel->type == PANEL_TYPE_LOCATE ?
-	  panel_locate.files[panel->curs] : ppanel_file->files[panel->curs];
-
-	if (pfe->file_type == FT_NA) {
-		addstr("no status information available");
-		clrtoeol();
-		return;
-	}
-
-	BLANK(print_fields(pfe,display.scrcols,layout_line));
 }
 
 /* warning message - can be used also in text mode */
@@ -1051,15 +1102,31 @@ draw_line_pfe(FILE_ENTRY *pfe)
 }
 
 static void
-draw_line_file(int ln)
+draw_line_bm(int ln)
 {
-	draw_line_pfe(ppanel_file->files[ln]);
+	putstr_trunc(USTR(panel_bm.bm[ln]),display.pancols,0);
 }
 
 static void
-draw_line_locate(int ln)
+draw_line_cfg(int ln)
 {
-	draw_line_pfe(panel_locate.files[ln]);
+	putstr_trunc(panel_cfg.config[ln].var,CFGVAR_LEN,0);
+	addstr(" = ");			/* 3 */
+	putstr_trunc(config_print_value(ln),display.pancols - CFGVAR_LEN - 3,0);
+}
+
+static void
+draw_line_compare(int ln)
+{
+	static const char *description[] = {
+		"0: name, size, type (symbolic links ok)",
+		"1: name, size, type",
+		"2: name, size, type, ownership+permissions",
+		"3: name, size, type, contents",
+		"4: name, size, type, ownership+permissions, contents",
+	};
+
+	putstr_trunc(description[ln],display.pancols,0);
 }
 
 static void
@@ -1078,41 +1145,6 @@ draw_line_compl(int ln)
 	putstr_trunc(SDSTR(pcc->str),display.pancols - 9,0);
 }
 
-static void
-draw_line_conf(int ln)
-{
-
-	static const char
-	  *normal_menu[] = { "Cancel", "Accept", "Accept+Save" },
-	  *admin_menu[] = { "Cancel+Quit", "Save+Quit" };
-
-	if (ln < panel_cfg.extra_lines) {
-		addstr(" -> ");			/* 4 */
-		putstr_trunc((clex_data.admin ? admin_menu : normal_menu)[ln],
-		  display.pancols - 4,0);
-	} else {
-		ln -= panel_cfg.extra_lines;
-		putstr_trunc(panel_cfg.config[ln].var,CFGVAR_LEN,0);
-		addstr(" = ");			/* 3 */
-		putstr_trunc(config_print_value(ln),
-		  display.pancols - CFGVAR_LEN - 3,0);
-	}
-}
-
-static void
-draw_line_dir_split(int ln)
-{
-	int len;
-
-	len = dir_split_len(ln);
-	if (len > display.pancols)
-		putstr_trunc(panel_dir_split.dirname,display.pancols,0);
-	else {
-		putstr_trunc(panel_dir_split.dirname,len,OPT_NOCONT);
-		BLANK(display.pancols - len);
-	}
-}
-
 static void
 draw_line_dir(int ln)
 {
@@ -1129,6 +1161,26 @@ draw_line_dir(int ln)
 	  display.pancols - shlen,0);
 }
 
+static void
+draw_line_dir_split(int ln)
+{
+	int len;
+
+	len = panel_dir_split.dir[ln].shlen;
+	if (len > display.pancols)
+		putstr_trunc(panel_dir_split.dir[ln].name,display.pancols,0);
+	else {
+		putstr_trunc(panel_dir_split.dir[ln].name,len,OPT_NOCONT);
+		BLANK(display.pancols - len);
+	}
+}
+
+static void
+draw_line_file(int ln)
+{
+	draw_line_pfe(ppanel_file->files[ln]);
+}
+
 static void
 draw_line_help(int ln)
 {
@@ -1151,6 +1203,12 @@ draw_line_hist(int ln)
 	putstr_trunc(USTR(panel_hist.hist[ln]->cmd),display.pancols - 8,0);
 }
 
+static void
+draw_line_locate(int ln)
+{
+	draw_line_pfe(panel_locate.files[ln]);
+}
+
 static void
 draw_line_mainmenu(int ln)
 {
@@ -1160,6 +1218,7 @@ draw_line_mainmenu(int ln)
 		"  change into root directory             <esc> /",
 		"  change into parent directory           <esc> .",
 		"  change into home directory             <esc> ~",
+		"  bookmarks                              <esc> B",
 		"command history                          <esc> H",
 		"sort order for filenames                 <esc> S",
 		"re-read current directory                ctrl-R",
@@ -1180,20 +1239,6 @@ draw_line_mainmenu(int ln)
 	putstr_trunc(description[ln],display.pancols,0);
 }
 
-static void
-draw_line_compare(int ln)
-{
-	static const char *description[] = {
-		"0: name, size, type (symbolic links ok)",
-		"1: name, size, type",
-		"2: name, size, type, ownership+permissions",
-		"3: name, size, type, contents",
-		"4: name, size, type, ownership+permissions, contents",
-	};
-
-	putstr_trunc(description[ln],display.pancols,0);
-}
-
 static void
 draw_line_sort(int ln)
 {
@@ -1213,10 +1258,48 @@ draw_line_sort(int ln)
 	putstr_trunc(description[ln],display.pancols - 12,0);
 }
 
+static void
+draw_panel_extra_line(int ln)
+{
+	const char *msg;
+	static const char
+	  *menu_cfg[]        = { "Cancel", "Accept", "Accept+Save" },
+	  *menu_cfg_admin[]  = { "Exit", "Save" },
+	  *menu_bm[]         = { 0, "Manage bookmarks",
+							 "Bookmark current working directory" },
+	  *menu_dir[]        = { 0, "Bookmarks" };
+
+	addstr("--> ");			/* 4 */
+	switch (panel->type) {
+		case PANEL_TYPE_BM:
+			msg = menu_bm[ln];
+			break;
+		case PANEL_TYPE_CFG:
+			msg = menu_cfg[ln];
+			break;
+		case PANEL_TYPE_CFG_ADMIN:
+			msg = menu_cfg_admin[ln];
+			break;
+		case PANEL_TYPE_DIR:
+			msg = menu_dir[ln];
+			break;
+		default:
+			msg = 0;
+	}
+	putstr_trunc(msg ? msg : "Leave this panel",display.pancols - 4,0);
+}
+
 static void
 draw_panel_line(int y)
 {
 	int ln;
+	/* must correspond with PANEL_TYPE_XXX */
+	static void (*draw_line[])(int) = {
+	  draw_line_bm, draw_line_bm, draw_line_cfg, draw_line_cfg,
+	  draw_line_compare, draw_line_compl, draw_line_dir,
+	  draw_line_dir_split, draw_line_file, draw_line_help, draw_line_hist,
+	  draw_line_locate, draw_line_mainmenu, draw_line_sort
+	};
 
 	move(2 + y,0);
 	ln = panel->top + y;
@@ -1234,10 +1317,9 @@ draw_panel_line(int y)
 	else
 		addstr("  ");
 	if (ln < 0)
-		putstr_trunc("--- (leave this panel / ctrl-C) ---",
-		  display.pancols,0);
+		draw_panel_extra_line(ln - panel->min);
 	else
-		(*draw_line)(ln);
+		(*draw_line[panel->type])(ln);
 	if (panel->curs == ln) {
 		addch(' ');
 		attrset(A_NORMAL);
@@ -1256,32 +1338,7 @@ draw_panel(int optimize)
 	if (panel->type != save_ptype) {
 		/* panel type has changed */
 		optimize = 0;
-		switch (save_ptype = panel->type) {
-		case PANEL_TYPE_CFG:
-			draw_line = draw_line_conf;		break;
-		case PANEL_TYPE_COMPARE:
-			draw_line = draw_line_compare;	break;
-		case PANEL_TYPE_COMPL:
-			draw_line = draw_line_compl;	break;
-		case PANEL_TYPE_DIR:
-			draw_line = draw_line_dir;		break;
-		case PANEL_TYPE_DIR_SPLIT:
-			draw_line = draw_line_dir_split;break;
-		case PANEL_TYPE_FILE:
-			draw_line = draw_line_file;		break;
-		case PANEL_TYPE_HELP:
-			draw_line = draw_line_help;		break;
-		case PANEL_TYPE_HIST:
-			draw_line = draw_line_hist;		break;
-		case PANEL_TYPE_LOCATE:
-			draw_line = draw_line_locate;	break;
-		case PANEL_TYPE_MAINMENU:
-			draw_line = draw_line_mainmenu;	break;
-		case PANEL_TYPE_SORT:
-			draw_line = draw_line_sort;		break;
-		}
-		if (!panel->infoline)	/* this is not a typo */
-			win_info();
+		save_ptype = panel->type;
 	}
 
 	if (optimize && save_top == panel->top) {
@@ -1290,6 +1347,7 @@ draw_panel(int optimize)
 		if (save_curs != panel->curs) {
 			pos_panel = 1;
 			draw_panel_line(panel->curs - panel->top);
+			save_curs = panel->curs;
 		}
 		else if (panel->type ==  PANEL_TYPE_FILE)
 			/* number of selected files could have changed */
@@ -1301,11 +1359,10 @@ draw_panel(int optimize)
 		for (y = 0; y < display.panlines; y++)
 			draw_panel_line(y);
 		save_top = panel->top;
+		save_curs = panel->curs;
 	}
-	save_curs = panel->curs;
 
-	if (panel->infoline)
-		win_info();
+	win_info();
 }
 
 /* win_panel() without optimization */
