@@ -44,7 +44,7 @@ lang_initialize(void)
 	struct lconv *lc;
 	time_t testtime;
 
-	testtime = 2916000L; /* 03-feb-1970 18:00 */
+	testtime = 2916000; /* 03-feb-1970 18:00 */
 
 	if (setlocale(LC_ALL,"") == 0)
 		txt_printf("LOCALE: cannot set program's locale\n");
