@@ -21,7 +21,6 @@
 #include "clex.h"
 #include "locate.h"
 
-#include "cfg.h"		/* config_num() */
 #include "control.h"	/* get_current_mode() */
 #include "edit.h"		/* edit_setprompt() */
 #include "panel.h"		/* pan_adjust() */
@@ -116,15 +115,13 @@ cx_locate_go(void)
 	int i;
 	FILE_ENTRY *pfe;
 
-	if (panel_locate.pd->cnt) {
-		pfe = panel_locate.files[panel_locate.pd->curs];
-		for (i = 0 ; i < ppanel_file->pd->cnt; i++)
-			if (ppanel_file->files[i] == pfe) {
-				ppanel_file->pd->curs = i;
-				pan_adjust(ppanel_file->pd);
-				break;
-			}
-	}
+	pfe = panel_locate.files[panel_locate.pd->curs];
+	for (i = 0 ; i < ppanel_file->pd->cnt; i++)
+		if (ppanel_file->files[i] == pfe) {
+			ppanel_file->pd->curs = i;
+			pan_adjust(ppanel_file->pd);
+			break;
+		}
 	next_mode = MODE_SPECIAL_RETURN;
 }
 
@@ -132,9 +129,6 @@ cx_locate_go(void)
 void
 cx_locate_tab(void)
 {
-	if (panel_locate.pd->cnt == 0)
-		return;
-
 	if (kbd_esc())
 		cx_edit_fullpath();
 	else
