@@ -29,7 +29,6 @@
 #include "inout.h"		/* win_panel() */
 #include "match.h"		/* match() */
 #include "list.h"		/* list_both_directories() */
-#include "panel.h"		/* pan_adjust() */
 #include "sdstring.h"	/* SDSTR() */
 #include "sort.h"		/* sort_files() */
 #include "ustring.h"	/* USTR() */
@@ -53,7 +52,6 @@ select_prepare(void)
 	edit_setprompt(&line_tmp,prompt);
 	textline = &line_tmp;
 	edit_nu_putstr("*");
-	edit_adjust();
 
 	if (TCLR(hint))
 		win_remark("wildcards: ? * and [..], see help");
@@ -64,7 +62,6 @@ compare_prepare(void)
 {
 	/* leave cursor position unchanged */
 	panel = panel_compare.pd;
-	pan_adjust(panel);
 	textline = 0;
 }
 
@@ -91,9 +88,6 @@ selectfile(FILE_ENTRY *pfe, int function)
 void
 cx_select_toggle(void)
 {
-	if (ppanel_file->pd->cnt == 0)
-		return;
-
 	ppanel_file->selected +=
 	  selectfile(ppanel_file->files[ppanel_file->pd->curs],FN_TOGGLE);
 
