@@ -31,18 +31,15 @@
 
 #include "cfg.h"		/* config_num() */
 #include "directory.h"	/* filepos_save() */
-#include "panel.h"		/* pan_adjust() */
 #include "sdstring.h"	/* SDSTR() */
 
-static int group_files, collate;
-
 /* result of CMP() is an int even if V1 and V2 are not */
 #define CMP(V1,V2) ((V1) == (V2) ? 0 : (V1) < (V2) ? -1 : 1)
 
 void
 sort_prepare(void)
 {
-	panel_sort.pd->top = -1;
+	panel_sort.pd->top = panel_sort.pd->min;
 	panel_sort.pd->curs = 0;	/* sort by name (default) */
 	panel = panel_sort.pd;
 	textline = 0;
@@ -51,8 +48,7 @@ sort_prepare(void)
 void
 cx_sort_set(void)
 {
-	if (panel_sort.pd->curs >= 0
-	  && panel_sort.order != panel_sort.pd->curs) {
+	if (panel_sort.order != panel_sort.pd->curs) {
 		panel_sort.order = panel_sort.pd->curs;
 		filepos_save();
 		sort_files();
@@ -106,7 +102,7 @@ sort_group(FILE_ENTRY *pfe)
 			return GROUP_DOTDOTDIR;
 		return GROUP_DIR;
 	}
-	if (group_files == 2) {
+	if (config_num(CFG_GROUP_FILES) == 2) {
 		if (type == FT_DEV_CHAR)
 			return GROUP_CDEV;
 		if (type == FT_DEV_BLOCK)
@@ -125,7 +121,7 @@ qcmp(const void *e1, const void *e2)
 	pfe2 = (*(FILE_ENTRY **)e2);
 
 	/* I. file type grouping */
-	if (group_files > 0) {
+	if (config_num(CFG_GROUP_FILES) > 0) {
 		group1 = sort_group(pfe1);
 		group2 = sort_group(pfe2);
 		cmp = group1 - group2;
@@ -133,7 +129,7 @@ qcmp(const void *e1, const void *e2)
 			return cmp;
 
 		/* special sorting for devices */
-		if (group_files == 2
+		if (config_num(CFG_GROUP_FILES) == 2
 		  && (group1 == GROUP_BDEV || group1 == GROUP_CDEV) ) {
 			cmp = major(pfe1->devnum) - major(pfe2->devnum);
 			if (cmp)
@@ -149,7 +145,7 @@ qcmp(const void *e1, const void *e2)
 	case SORT_EMAN:
 		return revstrcmp(SDSTR(pfe1->file),SDSTR(pfe2->file));
 	case SORT_SUFFIX:
-		cmp = (collate ? STRCOLL : strcmp)	/* selecting a function ! */
+		cmp = (config_num(CFG_COLLATION) ? STRCOLL : strcmp)
 		  (pfe1->extension,pfe2->extension);
 		break;
 	case SORT_TIME:
@@ -171,7 +167,7 @@ qcmp(const void *e1, const void *e2)
 		return cmp;
 
 	/* III. sort by file name */
-	return (collate ? STRCOLL : strcmp)
+	return (config_num(CFG_COLLATION) ? STRCOLL : strcmp)
 	  (SDSTR(pfe1->file),SDSTR(pfe2->file));
 }
 
@@ -181,8 +177,5 @@ sort_files(void)
 	if (ppanel_file->pd->cnt == 0)
 		return;
 
-	group_files = config_num(CFG_GROUP_FILES);
-	collate = config_num(CFG_COLLATION);
-	qsort(ppanel_file->files,ppanel_file->pd->cnt,
-	  sizeof(FILE_ENTRY *),qcmp);
+	qsort(ppanel_file->files,ppanel_file->pd->cnt,sizeof(FILE_ENTRY *),qcmp);
 }
