@@ -23,6 +23,7 @@
 
 #include "clex.h"
 
+#include "bookmarks.h"	/* bm_initialize() */
 #include "cfg.h"		/* config_initialize() */
 #include "completion.h"	/* completion_initialize() */
 #include "control.h"	/* control_loop() */
@@ -46,36 +47,38 @@
 PANEL_DESC *panel = 0;
 /*
  * initialization:
- *   cnt, top, curs, min, type, norev, infoline
+ *   cnt, top, curs, min, type, norev
  *   <panel specific data>
  */
+static PANEL_DESC pd_bm =
+  { 0,0,-2,-2,PANEL_TYPE_BM,0 };
 static PANEL_DESC pd_compl =
-  { 0,0,0,-1,PANEL_TYPE_COMPL,0,1 };
+  { 0,0,0,-1,PANEL_TYPE_COMPL,0 };
 static PANEL_DESC pd_cfg =
-  { 0,0,0, 0,PANEL_TYPE_CFG,0,1 };
+  { CFG_VARIABLES,0,0,0,PANEL_TYPE_CFG,0 };
 static PANEL_DESC pd_dir =
-  { 0,0,0,-1,PANEL_TYPE_DIR,0,0 };
+  { 0,0,0,-2,PANEL_TYPE_DIR,0 };
 static PANEL_DESC pd_dir_split =
-  { 0,0,0,-1,PANEL_TYPE_DIR_SPLIT,0,0 };
+  { 0,0,0,-1,PANEL_TYPE_DIR_SPLIT,0 };
 static PANEL_DESC pd_help =
-  { 0,0,0, 0,PANEL_TYPE_HELP,1,0 };
+  { 0,0,0, 0,PANEL_TYPE_HELP,1 };
 static PANEL_DESC pd_hist =
-  { 0,0,0,-1,PANEL_TYPE_HIST,0,0 };
+  { 0,0,0,-1,PANEL_TYPE_HIST,0 };
 static PANEL_DESC pd_locate =
-  { 0,0,0, 0,PANEL_TYPE_LOCATE,0,1 };
+  { 0,0,0, 0,PANEL_TYPE_LOCATE,0 };
 static PANEL_DESC pd_mainmenu =
-  /* 19 items in this menu */
-  { 19,-1,-1,-1,PANEL_TYPE_MAINMENU,0,0 };
+  /* 20 items in this menu */
+  { 20,-1,-1,-1,PANEL_TYPE_MAINMENU,0 };
 static PANEL_DESC pd_compare =
   /* 5 items in this menu */
-  {  5,-1,1,-1,PANEL_TYPE_COMPARE,0,0 };
+  {  5,-1,1,-1,PANEL_TYPE_COMPARE,0 };
 static PANEL_DESC pd_sort =
   /* 7 items in this menu */ 
-  {  7,0,0,-1,PANEL_TYPE_SORT,0,0 };
+  {  7,0,0,-1,PANEL_TYPE_SORT,0 };
+PANEL_BM panel_bm = { &pd_bm };
 PANEL_COMPL panel_compl = { &pd_compl };
-PANEL_CFG panel_cfg = { &pd_cfg ,0,0 };
-PANEL_DIR panel_dir = { &pd_dir };
-PANEL_DIR_SPLIT panel_dir_split = { &pd_dir_split };
+PANEL_CFG panel_cfg = { &pd_cfg };
+PANEL_DIR panel_dir = { &pd_dir }, panel_dir_split = { &pd_dir_split };
 PANEL_HELP panel_help = { &pd_help };
 PANEL_HIST panel_hist = { &pd_hist };
 PANEL_LOCATE panel_locate = { &pd_locate, 0,0 };
@@ -87,6 +90,7 @@ DISPLAY display = { 0,0,0,0,0,0,0 };
 CLEX_DATA clex_data;
 TEXTLINE *textline = 0, line_cmd, line_tmp, line_dir;
 CODE next_mode;
+const void *pcfg[CFG_VARIABLES];
 
 int
 main(int argc, char *argv[])
@@ -170,6 +174,7 @@ main(int argc, char *argv[])
 	help_initialize();
 	if (!clex_data.admin) {
 		/* all these features are not used in admin mode */
+		bm_initialize();
 		completion_initialize();
 		dir_initialize();
 		exec_initialize();
