@@ -103,6 +103,22 @@ us_resize(USTRING *pustr, size_t memreq)
 	}
 }
 
+/* quick alternative to copy */
+void
+us_xchg(USTRING *s1, USTRING *s2)
+{
+	char *xstr;
+	size_t xalloc;
+
+	xstr      = s1->USstr;
+	s1->USstr = s2->USstr;
+	s2->USstr = xstr;
+
+	xalloc      = s1->USalloc;
+	s1->USalloc = s2->USalloc;
+	s2->USalloc = xalloc;
+}
+
 void
 us_copy(USTRING *pustr, const char *src)
 {
