@@ -15,6 +15,8 @@
 #include <config.h>
 
 #include <sys/types.h>			/* clex.h */
+#include <sys/stat.h>			/* fstat() */
+#include <fcntl.h>				/* open() */
 #include <stdio.h>				/* sprintf() */
 #include <stdlib.h>				/* malloc() */
 #include <string.h>				/* strlen() */
@@ -31,10 +33,6 @@
 static USTRING sf_buff = { 0,0 };
 static size_t sf_dirlen;
 
-#ifndef SSIZE_MAX
-# define SSIZE_MAX INT_MAX
-#endif
-
 /* get rid of directory part in 'pathname' */
 const char
 *base_name(const char *pathname)
@@ -147,29 +145,9 @@ my_strerror(int errcode)
 }
 
 /*
- * read data from a descriptor (file, pipe, etc.)
- * returned value - same as read()
- *
- * note: error check should be (r == -1) and not (r < 0) !
+ * dequote backslash quoted text: 'src' -> 'dst'
+ * 'dst' buffer must be large enough
  */
-ssize_t
-read_fd(int fd, char *buff, size_t bytes)
-{
-	size_t total;
-	ssize_t rd;
-
-	for (total = 0; bytes > 0; total += rd, bytes -= rd) {
-		rd = read(fd,buff + total,
-		  bytes > SSIZE_MAX ? SSIZE_MAX : bytes);
-		if (rd == -1)	/* error */
-			return -1;
-		if (rd == 0)	/* EOF */
-			break;
-	}
-	return total;
-}
-
-/* dequote backslash quoted text: 'src' -> 'dst' */
 size_t
 dequote_txt(const char *src, size_t len, char *dst)
 {
@@ -186,3 +164,79 @@ dequote_txt(const char *src, size_t len, char *dst)
 
 	return j;
 }
+
+/*
+ * under certain condition can read() return fewer bytes than requested,
+ * this wrapper function handles it
+ *
+ * remember: error check should be (read() == -1) and not (read() < 0)
+ */
+ssize_t
+read_fd(int fd, char *buff, size_t bytes)
+{
+	size_t total;
+	ssize_t rd;
+
+	for (total = 0; bytes > 0; total += rd, bytes -= rd) {
+		rd = read(fd,buff + total,bytes);
+		if (rd == -1)	/* error */
+			return -1;
+		if (rd == 0)	/* EOF */
+			break;
+	}
+	return total;
+}
+
+/*
+ * read file into memory
+ * input: filename, *size = max size
+ * output (success): return value = allocated buffer, *size = actual size
+ * output (failure): return value = NULL, *errcode = error code
+ */
+char *
+read_file(const char *filename, size_t *size, int *errcode)
+{
+	int fd;
+	struct stat stbuf;
+	size_t filesize;	/* we do not need off_t here */
+	ssize_t rd;
+	char *buffer;
+
+	if ( (fd = open(filename,O_RDONLY)) < 0) {
+		*errcode = 1;	/* cannot open (errno is set) */
+		return 0;
+	}
+
+	fstat(fd,&stbuf);	/* cannot fail with valid descriptor */
+	if (stbuf.st_size > *size) {
+		close(fd);
+		*errcode = 2;	/* size too big */
+		return 0;
+	}
+
+	/* read into memory */
+	filesize = stbuf.st_size;
+	buffer = emalloc(filesize + 1);
+	rd = filesize ? read_fd(fd,buffer,filesize) : 0;
+	close(fd);
+
+	if (rd == -1) {
+		free(buffer);
+		*errcode = 3;	/* read error (errno is set) */
+		return 0;
+	}
+
+	if (filesize && buffer[filesize - 1] != '\n')
+		buffer[filesize++] = '\n';
+
+	*size = filesize;
+	return buffer;
+}
+
+time_t
+mod_time(const char *file)
+{
+	struct stat stbuf;
+
+	return stat(file,&stbuf) < 0 ? 0 : stbuf.st_mtime;
+}
