@@ -57,7 +57,8 @@ xterm_title_initialize(void)
 static const char
 *get_title(void)
 {
-	int fd[2], rd;
+	int fd[2];
+	ssize_t rd;
 	const char *wid;
 	char *p1, *p2, title[128];
 	pid_t pid;
@@ -116,17 +117,15 @@ static const char
 void
 xterm_title_reconfig(void)
 {
-	int xterm_feature;
 	const char *term;
 
 	enabled = 0;
-	xterm_feature = config_num(CFG_XTERM_TITLE);
 	/* 0 = disabled */
-	if (xterm_feature == 0)
+	if (config_num(CFG_XTERM_TITLE) == 0)
 		return;
 
 	/* 1 = automatic -> check $TERM for xterm, kterm or dtterm */
-	if (xterm_feature == 1) {
+	if (config_num(CFG_XTERM_TITLE) == 1) {
 		if ( (term = getenv("TERM")) == 0)
 			return;
 		if (strncmp(term,"xterm",5) && strncmp(term,"kterm",5)
